@@ -39,8 +39,13 @@ class Migrator(object):
         for sortable, field in enumerate(table, start=1):
             if self.db._ignore_field_case:
                 field_name = field.name.lower()
+                field_rname = field._rname.lower()
             else:
                 field_name = field.name
+                field_rname = field._rname
+            if self.dbengine == "oracle":
+                # Oracle needs all field names quoted to ensure consistent case
+                field_rname = self.dialect.quote(field_rname)
             field_type = field.type
             if isinstance(field_type, SQLCustomType):
                 ftype = field_type.native or field_type.type
@@ -81,6 +86,10 @@ class Migrator(object):
                         )
 
                 # must be PK reference or unique
+                rfield_rname = rfield._rname
+                if self.dbengine == "oracle":
+                    rfield_rname = self.dialect.quote(rfield_rname)
+
                 if (
                     not rfield.type.startswith(("reference", "big-reference"))
                     and getattr(rtable, "_primarykey", None)
@@ -96,6 +105,13 @@ class Migrator(object):
                         TFK[rtablename][rfieldname] = field_name
                     else:
                         fk = rtable._rname + " (" + rfield._rname + ")"
+                        if self.dbengine == "oracle":
+                            fk = (
+                                self.dialect.quote(rtable._rname)
+                                + " ("
+                                + rfield_rname
+                                + ")"
+                            )
                         ftype = ftype + types["reference FK"] % dict(
                             # should be quoted
                             constraint_name=constraint_name,
@@ -124,12 +140,14 @@ class Migrator(object):
                         real_referenced = (
                             referenced in db and db[referenced]._rname or referenced
                         )
+                    if self.dbengine == "oracle":
+                        real_referenced = self.dialect.quote(real_referenced)
                     rfield = db[referenced]._id
                     ftype_info = dict(
                         index_name=self.dialect.quote(field._raw_rname + "__idx"),
-                        field_name=field._rname,
+                        field_name=field_rname,
                         constraint_name=self.dialect.quote(constraint_name),
-                        foreign_key="%s (%s)" % (real_referenced, rfield._rname),
+                        foreign_key="%s (%s)" % (real_referenced, rfield_rname),
                         on_delete_action=field.ondelete,
                     )
                     ftype_info["null"] = (
@@ -202,7 +220,7 @@ class Migrator(object):
                 sortable=sortable,
                 type=str(field_type),
                 sql=ftype,
-                rname=field._rname,
+                rname=field_rname,
                 raw_rname=field._raw_rname,
             )
 
@@ -219,7 +237,7 @@ class Migrator(object):
             # Postgres - PostGIS:
             # geometry fields are added after the table has been created, not now
             if not (self.dbengine == "postgres" and field_type.startswith("geom")):
-                fields.append("%s %s" % (field._rname, ftype))
+                fields.append("%s %s" % (field_rname, ftype))
         other = ";"
 
         # backend-specific extensions to fields
@@ -245,6 +263,15 @@ class Migrator(object):
                     "Table %s has incompatible ON DELETE actions in multi-field foreign key."
                     % table._dalname
                 )
+            tfk_field_name = ", ".join(fkeys)
+            tfk_foreign_key = ", ".join(pkeys)
+            tfk_foreign_table = rtable._rname
+            if self.dbengine == "oracle":
+                tfk_field_name = ", ".join([self.dialect.quote(fkey) for fkey in fkeys])
+                tfk_foreign_key = ", ".join(
+                    [self.dialect.quote(pkey) for pkey in pkeys]
+                )
+                tfk_foreign_table = self.dialect.quote(rtable._rname)
             fields = (
                 fields
                 + ",\n    "
@@ -252,16 +279,21 @@ class Migrator(object):
                 % dict(
                     constraint_name=constraint_name,
                     table_name=table._rname,
-                    field_name=", ".join(fkeys),
-                    foreign_table=rtable._rname,
-                    foreign_key=", ".join(pkeys),
+                    field_name=tfk_field_name,
+                    foreign_table=tfk_foreign_table,
+                    foreign_key=tfk_foreign_key,
                     on_delete_action=on_delete[0],
                 )
             )
 
+        table_rname = table._rname
+        if self.dbengine == "oracle":
+            # must be explicitly quoted to preserve case
+            table_rname = self.dialect.quote(table_rname)
+
         if getattr(table, "_primarykey", None):
             query = "CREATE TABLE %s(\n    %s,\n    %s) %s" % (
-                table._rname,
+                table_rname,
                 fields,
                 self.dialect.primary_key(
                     ", ".join([table[pk]._rname for pk in table._primarykey])
@@ -269,7 +301,7 @@ class Migrator(object):
                 other,
             )
         else:
-            query = "CREATE TABLE %s(\n    %s\n)%s" % (table._rname, fields, other)
+            query = "CREATE TABLE %s(\n    %s\n)%s" % (table_rname, fields, other)
 
         uri = self.adapter.uri
         if uri.startswith("sqlite:///") or uri.startswith("spatialite:///"):
@@ -347,6 +379,8 @@ class Migrator(object):
 
     def _fix(self, item):
         k, v = item
+        if self.dbengine == "oracle" and "rname" in v:
+            v["rname"] = self.dialect.quote(v["rname"])
         if not isinstance(v, dict):
             v = dict(type="unknown", sql=v)
         if self.db._ignore_field_case is not True:
@@ -378,6 +412,11 @@ class Migrator(object):
         sql_fields = dict(map(self._fix, iteritems(sql_fields)))
         sql_fields_old = dict(map(self._fix, iteritems(sql_fields_old)))
         sql_fields_aux = dict(map(self._fix, iteritems(sql_fields_aux)))
+
+        table_rname = table._rname
+        if self.dbengine == "oracle":
+            table_rname = self.dialect.quote(table_rname)
+
         if db._debug:
             db.logger.debug("migrating %s to %s" % (sql_fields_old, sql_fields))
 
@@ -385,7 +424,7 @@ class Migrator(object):
         for key in sql_fields_old:
             if key not in keys:
                 keys.append(key)
-        new_add = self.dialect.concat_add(table._rname)
+        new_add = self.dialect.concat_add(table_rname)
 
         metadata_change = False
         sql_fields_current = copy.copy(sql_fields_old)
@@ -402,7 +441,7 @@ class Migrator(object):
                     query = [
                         "ALTER TABLE %s ADD %s %s;"
                         % (
-                            table._rname,
+                            table_rname,
                             sql_fields[key]["rname"],
                             sql_fields_aux[key]["sql"].replace(", ", new_add),
                         )
@@ -419,10 +458,10 @@ class Migrator(object):
                         tt = sql_fields_aux[key]["sql"].replace(", ", new_add)
                         query = [
                             "ALTER TABLE %s ADD %s %s;"
-                            % (table._rname, sql_fields[key]["rname"], tt),
+                            % (table_rname, sql_fields[key]["rname"], tt),
                             "UPDATE %s SET %s=%s;"
                             % (
-                                table._rname,
+                                table_rname,
                                 sql_fields[key]["rname"],
                                 sql_fields_old[key]["rname"],
                             ),
@@ -455,14 +494,14 @@ class Migrator(object):
                 tt = sql_fields_aux[key]["sql"].replace(", ", new_add)
                 query = [
                     "ALTER TABLE %s ADD %s %s;"
-                    % (table._rname, sql_fields[key]["rname"], tt),
+                    % (table_rname, sql_fields[key]["rname"], tt),
                     "UPDATE %s SET %s=%s;"
                     % (
-                        table._rname,
+                        table_rname,
                         sql_fields[key]["rname"],
                         sql_fields_old[key]["rname"],
                     ),
-                    drop_expr % (table._rname, sql_fields_old[key]["rname"]),
+                    drop_expr % (table_rname, sql_fields_old[key]["rname"]),
                 ]
                 metadata_change = True
             elif (
@@ -476,15 +515,15 @@ class Migrator(object):
                 tt = sql_fields_aux[key]["sql"].replace(", ", new_add)
                 key_tmp = self.dialect.quote(key + "__tmp")
                 query = [
-                    "ALTER TABLE %s ADD %s %s;" % (table._rname, key_tmp, tt),
+                    "ALTER TABLE %s ADD %s %s;" % (table_rname, key_tmp, tt),
                     "UPDATE %s SET %s=%s;"
-                    % (table._rname, key_tmp, sql_fields_old[key]["rname"]),
-                    drop_expr % (table._rname, sql_fields_old[key]["rname"]),
+                    % (table_rname, key_tmp, sql_fields_old[key]["rname"]),
+                    drop_expr % (table_rname, sql_fields_old[key]["rname"]),
                     "ALTER TABLE %s ADD %s %s;"
-                    % (table._rname, sql_fields[key]["rname"], tt),
+                    % (table_rname, sql_fields[key]["rname"], tt),
                     "UPDATE %s SET %s=%s;"
-                    % (table._rname, sql_fields[key]["rname"], key_tmp),
-                    drop_expr % (table._rname, key_tmp),
+                    % (table_rname, sql_fields[key]["rname"], key_tmp),
+                    drop_expr % (table_rname, key_tmp),
                 ]
                 metadata_change = True
             elif sql_fields[key] != sql_fields_old[key]:
